@@ -34,6 +34,8 @@
 #include "config.h"
 #include "callbacks.h"
 #include "print.h"
+#include "external_commands.h"
+#include "prefs.h"
 
 static void create_document_in_new_tab (const gchar *path, const gchar *text,
 		const gchar *title);
@@ -43,21 +45,11 @@ static void file_save (void);
 static gboolean close_all (void);
 static void set_title (void);
 static void set_undo_redo_sensitivity (void);
-static void run_compilation (gchar *title, gchar *command);
-static void view_document (gchar *title, gchar *doc_extension);
-static void convert_document (gchar *title, gchar *doc_extension,
-		gchar *command);
-static void add_action (gchar *title, gchar *command, gchar *command_output,
-		gboolean error);
 static void update_cursor_position_statusbar (void);
 static void scroll_to_cursor (void);
 static gboolean find_next_match (const gchar *what, GtkSourceSearchFlags flags,
 		gboolean backward, GtkTextIter *match_start, GtkTextIter *match_end);
-static void change_font_source_view (void);
-static void create_preferences (void);
-
-static GtkWidget *pref_dialog = NULL;
-static gboolean pref_changed = FALSE;
+static void free_latexila (void);
 
 void
 cb_new (void)
@@ -81,6 +73,10 @@ cb_open (void)
 			NULL
 	);
 
+	if (! latexila.prefs.file_chooser_dir_is_empty)
+		gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (dialog),
+				latexila.prefs.file_chooser_dir);
+
 	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
 	{
 		gchar *filename = gtk_file_chooser_get_filename (
@@ -93,6 +89,17 @@ cb_open (void)
 		g_free (filename);
 		g_free (uri);
 	}
+
+	/* save the current folder */
+	if (! latexila.prefs.file_chooser_dir_is_empty)
+		g_free (latexila.prefs.file_chooser_dir);
+
+	latexila.prefs.file_chooser_dir = gtk_file_chooser_get_current_folder_uri (
+			GTK_FILE_CHOOSER (dialog));
+
+	if (latexila.prefs.file_chooser_dir != NULL)
+		latexila.prefs.file_chooser_dir_is_empty = FALSE;
+
 	gtk_widget_destroy (dialog);
 }
 
@@ -127,7 +134,7 @@ cb_save_as (void)
 	// if the user click on cancel
 	if (! latexila.active_doc->saved)
 	{
-		(*latexila.active_doc) = doc_backup;
+		*latexila.active_doc = doc_backup;
 		set_title ();
 	}
 }
@@ -163,7 +170,8 @@ cb_close_tab (GtkWidget *widget, GtkWidget *child)
 		close_document (page_to_close);
 
 		if (gtk_notebook_get_n_pages (latexila.notebook) > 0)
-			latexila.active_doc = g_list_nth_data (latexila.all_docs, gtk_notebook_get_current_page (latexila.notebook));
+			latexila.active_doc = g_list_nth_data (latexila.all_docs,
+					gtk_notebook_get_current_page (latexila.notebook));
 		else
 			latexila.active_doc = NULL;
 
@@ -184,6 +192,8 @@ cb_quit (void)
 {
 	if (close_all ())
 	{
+		save_preferences (&latexila.prefs);
+		free_latexila ();
 		print_info ("Bye bye");
 		gtk_main_quit ();
 	}
@@ -280,37 +290,28 @@ cb_select_all (void)
 	gtk_text_buffer_select_range (buffer, &start, &end);
 }
 
-void
-cb_preferences (void)
-{
-	if (pref_dialog == NULL)
-		create_preferences ();
-	gtk_window_present (GTK_WINDOW (pref_dialog));
-}
-
 void
 cb_zoom_in (void)
 {
-	latexila.font_size += PANGO_SCALE;
-	pango_font_description_set_size (latexila.font_desc, latexila.font_size);
+	latexila.prefs.font_size += PANGO_SCALE;
+	pango_font_description_set_size (latexila.prefs.font_desc,
+			latexila.prefs.font_size);
 	change_font_source_view ();
 }
 
 void
 cb_zoom_out (void)
 {
-	latexila.font_size -= PANGO_SCALE;
-	pango_font_description_set_size (latexila.font_desc, latexila.font_size);
+	latexila.prefs.font_size -= PANGO_SCALE;
+	pango_font_description_set_size (latexila.prefs.font_desc,
+			latexila.prefs.font_size);
 	change_font_source_view ();
 }
 
 void
 cb_zoom_reset (void)
 {
-	gchar *font_string = g_key_file_get_string (latexila.key_file,
-            PROGRAM_NAME, "font", NULL);
-    latexila.font_desc = pango_font_description_from_string (font_string);
-    latexila.font_size = pango_font_description_get_size (latexila.font_desc);
+	set_current_font_prefs (&latexila.prefs);
 	change_font_source_view ();
 }
 
@@ -545,12 +546,20 @@ cb_latex (void)
 	if (latexila.active_doc == NULL)
 		return;
 
-	gchar *title = _("Compile (latex)");
-	gchar *command = g_strdup_printf ("latex -interaction=nonstopmode %s",
-			latexila.active_doc->path);
+	cb_save ();
 
-	run_compilation (title, command);
-	g_free (command);
+	gchar *title = _("Compile (latex)");
+	gchar *command[] = {
+        g_strdup (latexila.prefs.command_latex),
+        "-interaction=nonstopmode",
+        g_strdup (latexila.active_doc->path),
+        NULL
+    };
+
+    compile_document (title, command);
+
+    g_free (command[0]);
+    g_free (command[2]);
 }
 
 void
@@ -559,12 +568,20 @@ cb_pdflatex (void)
 	if (latexila.active_doc == NULL)
 		return;
 
-	gchar *title = _("Compile (pdflatex)");
-	gchar *command = g_strdup_printf ("pdflatex -interaction=nonstopmode %s",
-			latexila.active_doc->path);
+	cb_save ();
 
-	run_compilation (title, command);
-	g_free (command);
+	gchar *title = _("Compile (pdflatex)");
+	gchar *command[] = {
+        g_strdup (latexila.prefs.command_pdflatex),
+        "-interaction=nonstopmode",
+        g_strdup (latexila.active_doc->path),
+        NULL
+    };
+
+    compile_document (title, command);
+
+    g_free (command[0]);
+    g_free (command[2]);
 }
 
 void
@@ -600,7 +617,7 @@ cb_dvi_to_pdf (void)
 	if (latexila.active_doc == NULL)
 		return;
 
-	convert_document (_("DVI to PDF"), ".dvi", "dvipdf");
+	convert_document (_("DVI to PDF"), ".dvi", latexila.prefs.command_dvipdf);
 }
 
 void
@@ -609,7 +626,7 @@ cb_dvi_to_ps (void)
 	if (latexila.active_doc == NULL)
 		return;
 
-	convert_document (_("DVI to PS"), ".dvi", "dvips");
+	convert_document (_("DVI to PS"), ".dvi", latexila.prefs.command_dvips);
 }
 
 void
@@ -619,15 +636,12 @@ cb_action_list_changed (GtkTreeSelection *selection, gpointer user_data)
 	GtkTreeModel *model;
 	if (gtk_tree_selection_get_selected (selection, &model, &iter))
 	{
-		gchar *title, *command, *command_output;
-		gboolean error;
+		GtkTextBuffer *text_buffer;
 		gtk_tree_model_get (model, &iter,
-				COLUMN_ACTION_TITLE, &title,
-				COLUMN_ACTION_COMMAND, &command,
-				COLUMN_ACTION_COMMAND_OUTPUT, &command_output,
-				COLUMN_ACTION_ERROR, &error,
+				COLUMN_ACTION_TEXTBUFFER, &text_buffer,
 				-1);
-		print_log (latexila.log, title, command, command_output, error);
+
+		gtk_text_view_set_buffer (latexila.action_log.text_view, text_buffer);
 	}
 }
 
@@ -653,16 +667,24 @@ cb_about_dialog (void)
 	//TODO set the url hook
 	gchar *website = "http://latexila.sourceforge.net/";
 
-	const gchar * const authors[] =
+	const gchar *authors[] =
 	{
 		"Sébastien Wilmet <sebastien.wilmet@gmail.com>",
 		NULL
 	};
 
+	const gchar *artists[] =
+	{
+		"The Kile Team: http://kile.sourceforge.net/",
+		"Gedit LaTeX Plugin: http://www.michaels-website.de/gedit-latex-plugin/",
+		NULL
+	};
+
 	gtk_show_about_dialog (
 			latexila.main_window,
 			"program-name", PROGRAM_NAME,
 			"authors", authors,
+			"artists", artists,
 			"comments", comments,
 			"copyright", copyright,
 			"license", licence,
@@ -706,6 +728,16 @@ cb_page_change (GtkNotebook *notebook, GtkNotebookPage *page, guint page_num,
 	update_cursor_position_statusbar ();
 }
 
+void
+cb_page_reordered (GtkNotebook *notebook, GtkWidget *child, guint page_num,
+		gpointer user_data)
+{
+	// keep the same order in the GList all_docs than the tabs
+	latexila.all_docs = g_list_remove (latexila.all_docs, latexila.active_doc);
+	latexila.all_docs = g_list_insert (latexila.all_docs, latexila.active_doc,
+			page_num);
+}
+
 void
 cb_recent_item_activated (GtkRecentAction *action, gpointer user_data)
 {
@@ -718,53 +750,33 @@ cb_recent_item_activated (GtkRecentAction *action, gpointer user_data)
 }
 
 void
-cb_pref_dialog_close (GtkDialog *dialog, gint response_id, gpointer user_data)
+cb_show_symbol_tables (GtkToggleAction *toggle_action, gpointer user_data)
 {
-	gtk_widget_hide (GTK_WIDGET (dialog));
+	if (latexila.symbols.vbox == NULL)
+		return;
 
-	if (pref_changed)
-		save_preferences ();
+	latexila.prefs.show_side_pane =
+		gtk_toggle_action_get_active (toggle_action);
+
+	if (latexila.prefs.show_side_pane)
+		gtk_widget_show_all (latexila.symbols.vbox);
+	else
+		gtk_widget_hide (latexila.symbols.vbox);
 }
 
 void
-cb_line_numbers (GtkToggleButton *toggle_button, gpointer user_data)
+cb_show_edit_toolbar (GtkToggleAction *toggle_action, gpointer user_data)
 {
-	gboolean show_line_numbers = gtk_toggle_button_get_active (toggle_button);
-	g_key_file_set_boolean (latexila.key_file, PROGRAM_NAME,
-			"show_line_numbers", show_line_numbers);
+	if (latexila.edit_toolbar == NULL)
+		return;
 
-	//TODO optimisation?
-	guint nb_docs = g_list_length (latexila.all_docs);
-	for (guint i = 0 ; i < nb_docs ; i++)
-	{
-		document_t *doc = g_list_nth_data (latexila.all_docs, i);
-		gtk_source_view_set_show_line_numbers (
-				GTK_SOURCE_VIEW (doc->source_view), show_line_numbers);
-	}
+	latexila.prefs.show_edit_toolbar =
+		gtk_toggle_action_get_active (toggle_action);
 
-	pref_changed = TRUE;
-}
-
-void
-cb_command_view (GtkButton *button, gpointer user_data)
-{
-	GtkEntry *entry = (GtkEntry *) user_data;
-	const gchar *new_command_view = gtk_entry_get_text (entry);
-	g_key_file_set_string (latexila.key_file, PROGRAM_NAME,
-			"command_view", new_command_view);
-	pref_changed = TRUE;
-}
-
-void
-cb_font_set (GtkFontButton *font_button, gpointer user_data)
-{
-	const gchar *font_string = gtk_font_button_get_font_name (font_button);
-	g_key_file_set_string (latexila.key_file, PROGRAM_NAME,
-			"font", font_string);
-	latexila.font_desc = pango_font_description_from_string (font_string);
-	latexila.font_size = pango_font_description_get_size (latexila.font_desc);
-	change_font_source_view ();
-	pref_changed = TRUE;
+	if (latexila.prefs.show_edit_toolbar)
+		gtk_widget_show_all (latexila.edit_toolbar);
+	else
+		gtk_widget_hide (latexila.edit_toolbar);
 }
 
 void
@@ -787,34 +799,47 @@ open_new_document (const gchar *filename, const gchar *uri)
 		g_free (contents);
 		g_free (text_utf8);
 	}
+
 	else
+	{
 		print_warning ("impossible to open the file \"%s\"", filename);
+
+		GtkWidget *dialog = gtk_message_dialog_new (
+				latexila.main_window,
+				GTK_DIALOG_MODAL,
+				GTK_MESSAGE_WARNING,
+				GTK_BUTTONS_CLOSE,
+				_("Impossible to open the file \"%s\"."), filename);
+		gtk_dialog_run (GTK_DIALOG (dialog));
+		gtk_widget_destroy (dialog);
+	}
 }
 
 void
-save_preferences (void)
+change_font_source_view (void)
 {
-	gchar *key_file_data = g_key_file_to_data (latexila.key_file, NULL, NULL);
-	FILE* file = fopen (latexila.pref_file, "w");
-	if (file != NULL)
+	if (latexila.active_doc == NULL)
+		return;
+
+	// traverse the list
+	// an other solution is to call g_list_foreach ()
+	GList *current = latexila.all_docs;
+	do
 	{
-		fprintf (file, "%s", key_file_data);
-		fclose (file);
+		document_t *doc = g_list_nth_data (current, 0);
+		gtk_widget_modify_font (doc->source_view, latexila.prefs.font_desc);
 	}
-	else
-	{
-		print_warning ("impossible to save user preferences to \"%s\"",
-				latexila.pref_file);
-	}
-	g_free (key_file_data);
+	while ((current = g_list_next (current)) != NULL);
 }
 
+
 /******************************************************************************
  * local functions
  *****************************************************************************/
 
 static void
-create_document_in_new_tab (const gchar *path, const gchar *text, const gchar *title)
+create_document_in_new_tab (const gchar *path, const gchar *text,
+		const gchar *title)
 {
 	// create a new document_t structure
 	// if path = NULL, this is a new document
@@ -853,17 +878,16 @@ create_document_in_new_tab (const gchar *path, const gchar *text, const gchar *t
 	gtk_source_view_set_auto_indent (GTK_SOURCE_VIEW (new_doc->source_view), TRUE);
 
 	// set the font
-	gtk_widget_modify_font (new_doc->source_view, latexila.font_desc);
+	gtk_widget_modify_font (new_doc->source_view, latexila.prefs.font_desc);
 
 	// enable text wrapping (between words only)
 	gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (new_doc->source_view),
 			GTK_WRAP_WORD);
 
 	// show line numbers?
-	gboolean show_line_numbers = g_key_file_get_boolean (latexila.key_file,
-			PROGRAM_NAME, "show_line_numbers", NULL);
 	gtk_source_view_set_show_line_numbers (
-			GTK_SOURCE_VIEW (new_doc->source_view), show_line_numbers);
+			GTK_SOURCE_VIEW (new_doc->source_view),
+			latexila.prefs.show_line_numbers);
 
 	// put the text into the buffer
 	gtk_source_buffer_begin_not_undoable_action (new_doc->source_buffer);
@@ -891,7 +915,6 @@ create_document_in_new_tab (const gchar *path, const gchar *text, const gchar *t
 	gtk_widget_show_all (sw);
 
 	// add the new document in a new tab
-	// TODO set height
 	GtkWidget *hbox = gtk_hbox_new (FALSE, 3);
 
 	GtkWidget *label = gtk_label_new (title);
@@ -914,6 +937,7 @@ create_document_in_new_tab (const gchar *path, const gchar *text, const gchar *t
 	gtk_widget_show_all (hbox);
 
 	gint index = gtk_notebook_append_page (latexila.notebook, sw, hbox);
+	gtk_notebook_set_tab_reorderable (latexila.notebook, sw, TRUE);
 	gtk_notebook_set_current_page (latexila.notebook, index);
 	
 	set_undo_redo_sensitivity ();
@@ -930,22 +954,26 @@ close_document (gint index)
 	 * closing */
 	if (! latexila.active_doc->saved)
 	{
-		GtkWidget *dialog = gtk_dialog_new_with_buttons (
-				_("Close the document"),
+		gchar *doc_name;
+		if (latexila.active_doc->path == NULL)
+			doc_name = g_strdup (_("New document"));
+		else
+			doc_name = g_path_get_basename (latexila.active_doc->path);
+
+		GtkWidget *dialog = gtk_message_dialog_new (
 				latexila.main_window,
 				GTK_DIALOG_MODAL,
+				GTK_MESSAGE_QUESTION,
+				GTK_BUTTONS_NONE,
+				_("Save changes to \"%s\" before closing?"),
+				doc_name);
+		g_free (doc_name);
+
+		gtk_dialog_add_buttons (GTK_DIALOG (dialog),
 				GTK_STOCK_YES, GTK_RESPONSE_YES,
 				GTK_STOCK_NO, GTK_RESPONSE_NO,
 				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-				NULL
-		);
-
-		GtkWidget *label = gtk_label_new (_("Save changes to document before closing?"));
-		//gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), label, TRUE, FALSE, 10);
-		//gtk_widget_show_all (GTK_DIALOG (dialog)->vbox);
-		GtkWidget *content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
-		gtk_box_pack_start (GTK_BOX (content_area), label, TRUE, FALSE, 10);
-		gtk_widget_show_all (content_area);
+				NULL);
 
 		switch (gtk_dialog_run (GTK_DIALOG (dialog)))
 		{
@@ -987,12 +1015,73 @@ save_as_dialog (void)
 			latexila.main_window,
 			GTK_FILE_CHOOSER_ACTION_SAVE,
 			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-			GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+			GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
 			NULL
 	);
 
-	if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
-		latexila.active_doc->path = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
+	if (! latexila.prefs.file_chooser_dir_is_empty)
+		gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (dialog),
+				latexila.prefs.file_chooser_dir);
+
+	while (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
+	{
+		gchar *filename = gtk_file_chooser_get_filename (
+				GTK_FILE_CHOOSER (dialog));
+		
+		/* if the file exists, ask the user whether the file can be replaced */
+		if (g_file_test (filename, G_FILE_TEST_EXISTS))
+		{
+			GtkWidget *confirmation = gtk_message_dialog_new (
+					latexila.main_window,
+					GTK_DIALOG_MODAL,
+					GTK_MESSAGE_QUESTION,
+					GTK_BUTTONS_NONE,
+					_("A file named \"%s\" already exists. Do you want to replace it?"),
+					g_path_get_basename (filename));
+
+			gtk_dialog_add_button (GTK_DIALOG (confirmation),
+					GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
+
+			// button replace created by hand because we use the icon "save as"
+			// with the text "replace"
+			GtkWidget *button_replace = gtk_button_new_with_label (_("Replace"));
+			GtkWidget *icon = gtk_image_new_from_stock (GTK_STOCK_SAVE_AS,
+					GTK_ICON_SIZE_BUTTON);
+			gtk_button_set_image (GTK_BUTTON (button_replace), icon);
+			gtk_dialog_add_action_widget (GTK_DIALOG (confirmation),
+					button_replace, GTK_RESPONSE_YES);
+			gtk_widget_show (button_replace);
+
+			gint response = gtk_dialog_run (GTK_DIALOG (confirmation));
+			gtk_widget_destroy (confirmation);
+
+			// return to the file chooser dialog so the user can choose an other
+			// file
+			if (response != GTK_RESPONSE_YES)
+			{
+				g_free (filename);
+				continue;
+			}
+		}
+
+		latexila.active_doc->path = filename;
+		gchar *uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (dialog));
+
+		GtkRecentManager *manager = gtk_recent_manager_get_default ();
+		gtk_recent_manager_add_item (manager, uri);
+
+		break;
+	}
+
+	/* save the current folder */
+	if (! latexila.prefs.file_chooser_dir_is_empty)
+		g_free (latexila.prefs.file_chooser_dir);
+
+	latexila.prefs.file_chooser_dir = gtk_file_chooser_get_current_folder_uri (
+			GTK_FILE_CHOOSER (dialog));
+
+	if (latexila.prefs.file_chooser_dir != NULL)
+		latexila.prefs.file_chooser_dir_is_empty = FALSE;
 
 	gtk_widget_destroy (dialog);
 }
@@ -1003,34 +1092,35 @@ file_save (void)
 	if (latexila.active_doc->path == NULL)
 		return;
 
+	GError *error = NULL;
 	print_info ("save current buffer to \"%s\"", latexila.active_doc->path);
 
-	FILE* file = fopen (latexila.active_doc->path, "w");
-	if (file != NULL)
+	GtkTextBuffer *text_buffer = GTK_TEXT_BUFFER (latexila.active_doc->source_buffer);
+	GtkTextIter start;
+	GtkTextIter end;
+	gtk_text_buffer_get_bounds (text_buffer, &start, &end);
+	gchar *contents = gtk_text_buffer_get_text (text_buffer, &start, &end, FALSE);
+	gchar *locale = g_locale_from_utf8 (contents, -1, NULL, NULL, &error);
+	g_free (contents);
+
+	if (error != NULL)
 	{
-		GtkTextBuffer *text_buffer = GTK_TEXT_BUFFER (latexila.active_doc->source_buffer);
-		GtkTextIter start;
-		GtkTextIter end;
-		gtk_text_buffer_get_bounds (text_buffer, &start, &end);
-		gchar *contents = gtk_text_buffer_get_text (text_buffer, &start, &end, FALSE);
-		gchar *locale = g_locale_from_utf8 (contents, -1, NULL, NULL, NULL);
-		
-		// write the contents into the file
-		fprintf (file, "%s\n", locale);
-
-		fclose (file);
-		g_free (contents);
-		g_free (locale);
-
-		latexila.active_doc->saved = TRUE;
+		print_warning ("impossible to convert the contents: %s", error->message);
+		g_error_free (error);
+		return;
 	}
 
-	// error with fopen
-	else
+	g_file_set_contents (latexila.active_doc->path, locale, -1, &error);
+
+	if (error != NULL)
 	{
-		print_warning ("impossible to save the file \"%s\"",
-				latexila.active_doc->path);
+		print_warning ("impossible to save the file: %s", error->message);
+		g_error_free (error);
+		return;
 	}
+
+	latexila.active_doc->saved = TRUE;
+	g_free (locale);
 }
 
 static gboolean
@@ -1038,7 +1128,7 @@ close_all (void)
 {
 	while (latexila.active_doc != NULL)
 	{
-		int tmp = gtk_notebook_get_n_pages (latexila.notebook);
+		gint tmp = gtk_notebook_get_n_pages (latexila.notebook);
 		gtk_notebook_set_current_page (latexila.notebook, 0);
 		cb_close ();
 		if (gtk_notebook_get_n_pages (latexila.notebook) >= tmp)
@@ -1085,244 +1175,8 @@ set_undo_redo_sensitivity (void)
 		can_redo = gtk_source_buffer_can_redo (latexila.active_doc->source_buffer);
 	}
 
-	gtk_action_set_sensitive (latexila.undo, can_undo);
-	gtk_action_set_sensitive (latexila.redo, can_redo);
-}
-
-static void
-run_compilation (gchar *title, gchar *command)
-{
-	if (latexila.active_doc == NULL)
-		return;
-
-	gchar *command_output;
-	gboolean is_error = TRUE;
-
-	/* the current document is a *.tex file? */
-	gboolean tex_file = g_str_has_suffix (latexila.active_doc->path, ".tex");
-	if (! tex_file)
-	{
-		command_output = g_strdup_printf (_("compilation failed: %s is not a *.tex file"),
-				g_path_get_basename (latexila.active_doc->path));
-
-		add_action (title, command, command_output, is_error);
-		g_free (command_output);
-		return;
-	}
-
-	/* print a message in the statusbar */
-	guint context_id = gtk_statusbar_get_context_id (latexila.statusbar,
-			"running-action");
-	gtk_statusbar_push (latexila.statusbar, context_id,
-			_("Compilation in progress. Please wait..."));
-
-	// without that, the message in the statusbar does not appear
-	while (gtk_events_pending ())
-		gtk_main_iteration ();
-
-	/* run the command in the directory where the .tex is */
-	gchar *dir_backup = g_get_current_dir ();
-	gchar *dir = g_path_get_dirname (latexila.active_doc->path);
-	g_chdir (dir);
-
-	print_info ("execution of the command: %s", command);
-	
-	GError *error = NULL;
-	g_spawn_command_line_sync (command, &command_output, NULL, NULL, &error);
-	g_chdir (dir_backup);
-	
-	// an error occured
-	if (error != NULL)
-	{
-		command_output = g_strdup_printf (_("execution failed: %s"),
-				error->message);
-		g_error_free (error);
-	}
-	else
-		is_error = FALSE;
-
-	add_action (title, command, command_output, is_error);
-
-	gtk_statusbar_pop (latexila.statusbar, context_id);
-
-	g_free (command_output);
-	g_free (dir_backup);
-	g_free (dir);
-}
-
-static void
-view_document (gchar *title, gchar *doc_extension)
-{
-	if (latexila.active_doc == NULL)
-		return;
-
-	gchar *command, *command_output;
-	gboolean is_error = TRUE;
-
-	GError *error = NULL;
-	GRegex *regex = g_regex_new ("\\.tex$", 0, 0, NULL);
-
-	/* replace .tex by doc_extension (.pdf, .dvi, ...) */
-	gchar *doc_path = g_regex_replace_literal (regex, latexila.active_doc->path,
-			-1, 0, doc_extension, 0, NULL);
-
-	gchar *command_view = g_key_file_get_string (latexila.key_file, PROGRAM_NAME,
-			"command_view", NULL);
-	command = g_strdup_printf ("%s %s", command_view, doc_path);
-
-	/* the current document is a *.tex file? */
-	gboolean tex_file = g_regex_match (regex, latexila.active_doc->path, 0, NULL);
-	if (! tex_file)
-	{
-		command_output = g_strdup_printf (_("failed: %s is not a *.tex file"),
-				g_path_get_basename (latexila.active_doc->path));
-
-		add_action (title, command, command_output, is_error);
-		g_free (command);
-		g_free (command_output);
-		g_free (doc_path);
-		g_regex_unref (regex);
-		return;
-	}
-
-	/* the document (PDF, DVI, ...) file exist? */
-	if (! g_file_test (doc_path, G_FILE_TEST_IS_REGULAR))
-	{
-		command_output = g_strdup_printf (
-				_("%s does not exist. If this is not already made, compile the document with the right command."),
-				g_path_get_basename (doc_path));
-
-		add_action (title, command, command_output, is_error);
-		g_free (command);
-		g_free (command_output);
-		g_free (doc_path);
-		g_regex_unref (regex);
-		return;
-	}
-
-	/* run the command */
-	print_info ("execution of the command: %s", command);
-	g_spawn_command_line_async (command, &error);
-
-	if (error != NULL)
-	{
-		command_output = g_strdup_printf (_("execution failed: %s"),
-				error->message);
-		g_error_free (error);
-	}
-	else
-	{
-		command_output = g_strdup (_("Viewing in progress. Please wait..."));
-		is_error = FALSE;
-	}
-
-	add_action (title, command, command_output, is_error);
-
-	g_free (command);
-	g_free (command_output);
-	g_free (doc_path);
-	g_regex_unref (regex);
-}
-
-static void
-convert_document (gchar *title, gchar *doc_extension, gchar *command)
-{
-	if (latexila.active_doc == NULL)
-		return;
-
-	gchar *full_command, *command_output;
-	gboolean is_error = TRUE;
-
-	GError *error = NULL;
-	GRegex *regex = g_regex_new ("\\.tex$", 0, 0, NULL);
-
-	/* replace .tex by doc_extension (.pdf, .dvi, ...) */
-	gchar *doc_path = g_regex_replace_literal (regex,
-			latexila.active_doc->path, -1, 0, doc_extension, 0, NULL);
-
-	full_command = g_strdup_printf ("%s %s", command, doc_path);
-
-	/* the document to convert exist? */
-	if (! g_file_test (doc_path, G_FILE_TEST_IS_REGULAR))
-	{
-		command_output = g_strdup_printf (
-				_("%s does not exist. If this is not already made, compile the document with the right command."),
-				g_path_get_basename (doc_path));
-
-		add_action (title, full_command, command_output, is_error);
-		g_free (full_command);
-		g_free (command_output);
-		g_free (doc_path);
-		g_regex_unref (regex);
-		return;
-	}
-
-	/* print a message in the statusbar */
-	guint context_id = gtk_statusbar_get_context_id (latexila.statusbar,
-			"running-action");
-	gtk_statusbar_push (latexila.statusbar, context_id,
-			_("Converting in progress. Please wait..."));
-
-	// without that, the message in the statusbar does not appear
-	while (gtk_events_pending ())
-		gtk_main_iteration ();
-
-	/* run the command in the directory where the .tex is */
-	print_info ("execution of the command: %s", full_command);
-
-	gchar *dir_backup = g_get_current_dir ();
-	gchar *dir = g_path_get_dirname (latexila.active_doc->path);
-	g_chdir (dir);
-	g_spawn_command_line_sync (full_command, &command_output, NULL, NULL, &error);
-	g_chdir (dir_backup);
-	
-	// an error occured
-	if (error != NULL)
-	{
-		command_output = g_strdup_printf (_("execution failed: %s"),
-				error->message);
-		g_error_free (error);
-	}
-	else
-		is_error = FALSE;
-
-	add_action (title, full_command, command_output, is_error);
-
-	gtk_statusbar_pop (latexila.statusbar, context_id);
-
-	g_free (full_command);
-	g_free (command_output);
-	g_free (doc_path);
-	g_regex_unref (regex);
-}
-
-static void
-add_action (gchar *title, gchar *command, gchar *command_output, gboolean error)
-{
-	static gint num = 1;
-	gchar *title2 = g_strdup_printf ("%d. %s", num, title);
-
-	// append an new entry to the action list
-	GtkTreeIter iter;
-	gtk_list_store_append (latexila.list_store, &iter);
-	gtk_list_store_set (latexila.list_store, &iter,
-			COLUMN_ACTION_TITLE, title2,
-			COLUMN_ACTION_COMMAND, command,
-			COLUMN_ACTION_COMMAND_OUTPUT, command_output,
-			COLUMN_ACTION_ERROR, error,
-			-1);
-
-	// the new entry is selected
-	// cb_action_list_changed () is called, so the details are showed
-	gtk_tree_selection_select_iter (latexila.list_selection, &iter);
-
-	// scroll to the end
-	GtkTreePath *path = gtk_tree_model_get_path (
-			GTK_TREE_MODEL (latexila.list_store), &iter);
-	gtk_tree_view_scroll_to_cell (latexila.list_view, path, NULL, FALSE, 0, 0);
-
-	num++;
-	g_free (title2);
+	gtk_action_set_sensitive (latexila.actions.undo, can_undo);
+	gtk_action_set_sensitive (latexila.actions.redo, can_redo);
 }
 
 static void
@@ -1456,70 +1310,18 @@ find_next_match (const gchar *what, GtkSourceSearchFlags flags,
 }
 
 static void
-change_font_source_view (void)
+free_latexila (void)
 {
-	guint nb_docs = g_list_length (latexila.all_docs);
-	for (guint i = 0 ; i < nb_docs ; i++)
-	{
-		document_t *doc = g_list_nth_data (latexila.all_docs, i);
-		gtk_widget_modify_font (doc->source_view, latexila.font_desc);
-	}
-}
-
-static void
-create_preferences (void)
-{
-	pref_dialog = gtk_dialog_new_with_buttons (_("Preferences"),
-			latexila.main_window, 0,
-			GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, NULL);
-	gtk_dialog_set_has_separator (GTK_DIALOG (pref_dialog), FALSE);
-
-	g_signal_connect (G_OBJECT (pref_dialog), "response",
-			G_CALLBACK (cb_pref_dialog_close), NULL);
-	g_signal_connect (G_OBJECT (pref_dialog), "destroy",
-			G_CALLBACK (gtk_widget_destroyed), &pref_dialog);
-
-	GtkWidget *content_area = gtk_dialog_get_content_area (
-			GTK_DIALOG (pref_dialog));
-	gtk_box_set_spacing (GTK_BOX (content_area), 3);
-
-	/* show line numbers */
-	GtkWidget *line_numbers = gtk_check_button_new_with_label (
-			_("Display line numbers"));
-	gboolean tmp = g_key_file_get_boolean (latexila.key_file, PROGRAM_NAME,
-			"show_line_numbers", NULL);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (line_numbers), tmp);
-	g_signal_connect (G_OBJECT (line_numbers), "toggled",
-			G_CALLBACK (cb_line_numbers), NULL);
-	gtk_box_pack_start (GTK_BOX (content_area), line_numbers, FALSE, FALSE, 0);
-
-	/* font */
-	GtkWidget *hbox = gtk_hbox_new (FALSE, 5);
-	GtkWidget *label = gtk_label_new (_("Font:"));
-	gchar *current_font = g_key_file_get_string (latexila.key_file, PROGRAM_NAME,
-			"font", NULL);
-	GtkWidget *font_button = gtk_font_button_new_with_font (current_font);
-	g_signal_connect (G_OBJECT (font_button), "font-set",
-			G_CALLBACK (cb_font_set), NULL);
-	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (hbox), font_button, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (content_area), hbox, FALSE, FALSE, 0);
-
-	/* command view entry */
-	hbox = gtk_hbox_new (FALSE, 5);
-	label = gtk_label_new (_("Program for viewing documents:"));
-	GtkWidget *command_view_entry = gtk_entry_new ();
-	gchar *txt = g_key_file_get_string (latexila.key_file, PROGRAM_NAME,
-			"command_view", NULL);
-	gtk_entry_set_text (GTK_ENTRY (command_view_entry), txt);
-	GtkWidget *button = gtk_button_new_with_label (_("OK"));
-	g_signal_connect (G_OBJECT (button), "clicked",
-			G_CALLBACK (cb_command_view), GTK_ENTRY (command_view_entry));
-	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (hbox), command_view_entry, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (content_area), hbox, FALSE, FALSE, 0);
-
-
-	gtk_widget_show_all (content_area);
+	g_free (latexila.prefs.font_str);
+	g_free (latexila.prefs.command_view);
+	g_free (latexila.prefs.command_latex);
+	g_free (latexila.prefs.command_pdflatex);
+	g_free (latexila.prefs.command_dvipdf);
+	g_free (latexila.prefs.command_dvips);
+
+	if (! latexila.prefs.file_chooser_dir_is_empty)
+		g_free (latexila.prefs.file_chooser_dir);
+
+	for (int i = 0 ; i < 7 ; i++)
+		g_object_unref (latexila.symbols.list_stores[i]);
 }
